package App::PerlPackage2PlantUMLClassDiagram;
use 5.014;
use strict;
use warnings;

our $VERSION = "0.03";

1;
__END__

=encoding utf-8

=head1 NAME

App::PerlPackage2PlantUMLClassDiagram - Generate Plant UML's class diagraom from Perl packages.

=head1 DESCRIPTION

App::PerlPackage2PlantUMLClassDiagram is backend of package2plantumlclassdiagram.

=head1 LICENSE

Copyright (C) hitode909.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 AUTHOR

hitode909 E<lt>hitode909@gmail.comE<gt>

=cut

